--[[ CombatTime ]]

if not FixR then return end

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
  ["showCombatLogging"] = false;
  ["combatLoggingReminder"] = true;
  ["continueEncounterWhenDead"] = true;
  ["useDBMPullDetection"] = true;
}
local misc = {
  ["desc"] = "Display combat length";
  ["alpha"] = true;
  ["GUI"] = {
    [1] = {
      ["type"] = "Boolean";
      ["opt"] = "showCombatLogging";
      ["desc"] = "display combat logging state";
    };
    [2] = {
      ["type"] = "Boolean";
      ["opt"] = "combatLoggingReminder";
      ["desc"] = "remind me to enable logging when entering an instance";
    };
    [3] = {
      ["type"] = "Boolean";
      ["opt"] = "continueEncounterWhenDead";
      ["desc"] = "continue to count when your character died but the encounter is still in progress";
    };
    [4] = {
      ["type"] = "Boolean";
      ["opt"] = "useDBMPullDetection";
      ["desc"] = "also use DBM's pull detection to start a combat session";
    };
    [5] = {
      ["type"] = "Button";
      ["opt"] = "toggle moving";
      ["func"] = function(self) self:SetOption("move"); end;
    };
    [6] = {
      ["type"] = "Button";
      ["opt"] = "reset position";
      ["func"] = function(self) self:SetOption("reset"); end;
    };
  };
}

local M = FixR:NewModule("CombatTime", DefaultO, misc)

local frame = nil
local frameEvents = {};
local DBMEventHandler = {};
local moving = false
local absorbAmount = 0

local function initFrame()
  frame = CreateFrame("Frame", "FixR_CombatTimeFrame", UIParent)
  frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  frame:SetFrameStrata("HIGH")
  frame:SetSize(220, 40)
  frame:SetScript("OnEvent", function(self, event, ...)
    frameEvents[event](self, ...)
  end);
  frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
  frame:SetScript("OnDragStop", function(self)
    self:StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
    M.O["framePoint"] = point or "CENTER"
    M.O["frameRelativeTo"] = relativeTo or "UIParent"
    M.O["frameRelativePoint"] = relativePoint or "CENTER"
    M.O["frameOffsetX"] = FixR:Round(xOfs, 4)
    M.O["frameOffsetY"] = FixR:Round(yOfs, 4)
  end);
  frame:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
  frame.timeSinceLastUpdate = 0
  frame.needsUpdate = false
  frame.inCombat = false
  frame.encounterInProgress = false
  frame.secondsInCombat = 0
  
  --CHAT_MSG_ADDON
  frame.COMprefix = "FixRCombatTime"
  frame.COMchannel = "RAID"
  frame.COMPlayerName = UnitName("player")
  frame.COMlastStatusRequestSent = -1
  local COMok = RegisterAddonMessagePrefix(frame.COMprefix)
  --FixR:Echo("CombatTime: Registering prefix... ", COMok and "|cffaaffaasuccessful" or "|cffff8888failed", ".")
  
  frame.combatLogging = LoggingCombat() and true or false
  
  frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
  frame.text:SetPoint("RIGHT")
  frame.text:SetText("CombatTime")
  frame.text:SetTextColor(1, 1, 1, 1)
  
  frame.startEncounter = function()
    frame.encounterInProgress = true
    frame.secondsInCombat = 0
    frame.timeSinceLastUpdate = 0
  end
  
  frame.OnUpdate = function(self, elapsed)
    self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed
    while (self.encounterInProgress and self.timeSinceLastUpdate > 1) do --in seconds
      self.secondsInCombat = self.secondsInCombat + 1
      self.needsUpdate = true
      if self.inCombat or M.O["continueEncounterWhenDead"] and IsEncounterInProgress() then
        self.encounterInProgress = true
      else
        self.encounterInProgress = false
      end
      
      self.timeSinceLastUpdate = self.timeSinceLastUpdate - 1
    end
    
    if self.needsUpdate == true then
      self.needsUpdate = false
      self.updateDisplay()
    end
    
    if (self.COMlastStatusRequestSent >= 0) and (self.COMlastStatusRequestSent < (time() - 10)) then --wait 10 sec for reply then ask anyway
      self.COMlastStatusRequestSent = -1
      self:checkLogging()
    end
    
  end
  frame.updateDisplay = function()
    local l = frame.secondsInCombat
    local d = math.floor(l / 86400)
    l = l - 86400 * d
    local h = math.floor(l / 3600)
    l = l - 3600 * h
    local m = math.floor(l / 60)
    l = l - 60 * m
    local s = l
    
    local t
    
    if d > 0 then
      if m < 10 then
        m = "0"..m
      end
      if s < 10 then
        s = "0"..s
      end
      t = string.format("%dd %d:%s:%s", d, h, m, s)
    elseif h > 0 then
      if m < 10 then
        m = "0"..m
      end
      if s < 10 then
        s = "0"..s
      end
      t = string.format("%d:%s:%s", h, m, s)
    elseif m > 0 then
      if s < 10 then
        s = "0"..s
      end
      t = string.format("%d:%s", m, s)
    else
      t = s
    end
    if M.O["showCombatLogging"] then
      if frame.combatLogging then
        t = t.." |cff00ff00ON"
      else
        t = t.." |cffff0000OFF"
      end
    end
    frame.text:SetText(t)
  end
  frame.combatLoggingEvent = function(msg)
    --isLogging = LoggingCombat([newState])
    frame.combatLogging = LoggingCombat() and true or false
    frame.updateDisplay()
  end
  hooksecurefunc(SlashCmdList, "COMBATLOG", frame.combatLoggingEvent)
  
  
  --Chat hyperlink stuff
  frame.MyOnHyperlinkShow = function(chatFrame, link, text, button)
    if link:sub(1, 15) ~= "FixRCombatTime:" then
      return
    end
    local info = ChatTypeInfo["SYSTEM"]
    if link:sub(16) == "LogYES" then
      LoggingCombat(true)
      DEFAULT_CHAT_FRAME:AddMessage(COMBATLOGENABLED, info.r, info.g, info.b, info.id)
    elseif link:sub(16) == "LogNO" then
      LoggingCombat(false)
      DEFAULT_CHAT_FRAME:AddMessage(COMBATLOGDISABLED, info.r, info.g, info.b, info.id)
    end
    frame.combatLoggingEvent()
  end
  hooksecurefunc("ChatFrame_OnHyperlinkShow", frame.MyOnHyperlinkShow)
  local old_SetHyperlink = ItemRefTooltip.SetHyperlink -- http://forums.wowace.com/showthread.php?t=20217
	function ItemRefTooltip:SetHyperlink(link, ...)
		if link:sub(1, 15) == "FixRCombatTime:" then
			return
		end
		return old_SetHyperlink(self, link, ...)
	end
  
  function frame:checkLogging()
    if not self.lastCheckLogging then self.lastCheckLogging = time()-10 end
    if self.lastCheckLogging > (time() - 10) then
      return
    else
      self.lastCheckLogging = time()
    end
    
    self.COMlastStatusRequestSent = -1
    
    if M.O["combatLoggingReminder"] then
      if not LoggingCombat() then
        FixR:Echo("Not logging combat, start now? \124cffaaffaa\124HFixRCombatTime:LogYES\124h[yes]\124h", " or \124cffff8888\124HFixRCombatTime:LogNO\124h[no]\124h")
      --else
      --  FixR:Echo("CombatTime: |cff00ff00Logging combat.")
      end
    end
  end
  
  function frame:COMrequestStatus()
    if M.O["combatLoggingReminder"] then
      if not LoggingCombat() then
        self.COMlastStatusRequestSent = time()
        SendAddonMessage(self.COMprefix, "statusreq", self.COMchannel)
      end
    end
  end
end

--[[
function frameEvents:UNIT_ABSORB_AMOUNT_CHANGED(unit)
  if unit == "player" then
    absorbAmount = UnitGetTotalAbsorbs("player")
    self.updateDisplay()
  end
end
--]]
function frameEvents:PLAYER_REGEN_DISABLED(...)
  if self.encounterInProgress == false then
    self.startEncounter()
  end
  self.inCombat = true
  self.updateDisplay()
end
function frameEvents:PLAYER_REGEN_ENABLED(...)
  self.inCombat = false
  self.updateDisplay()
end
function frameEvents:ZONE_CHANGED_NEW_AREA(...)
  local inInstance, instanceType = IsInInstance()
  if inInstance then
    --self:checkLogging()
    --request status when entering an instance
    self:COMrequestStatus()
  end
end
--function frameEvents:GROUP_ROSTER_UPDATE(...)
  --print("GROUP_ROSTER_UPDATE")
  --self:checkLogging()
--end
--[[
function frameEvents:PARTY_MEMBERS_CHANGED(...)
end
function frameEvents:RAID_ROSTER_UPDATE(...)
end
--]]
function frameEvents:CHAT_MSG_ADDON(prefix, msg, channel, from)
  if prefix == self.COMprefix then
    if (msg == "statusreq") and (from ~= self.COMPlayerName) then
      if LoggingCombat() then
        --disable answer for now, since it's pretty much redundant
        --SendAddonMessage(self.COMprefix, "LogON", "WHISPER", from)
      end
    elseif msg == "LogON" then
      self:checkLogging()
    end
  end
end

function DBMEventHandler:pull(event, mod, delay, synced)
  if M.O["useDBMPullDetection"] then
    --if frame.encounterInProgress == false then
      --let DBM:pull always reset the combat session
      frame.startEncounter()
    --end
    frame.updateDisplay()
  end
end



function M:Load()
  if not frame then
    initFrame()
  end
  for k, v in pairs(frameEvents) do
    frame:RegisterEvent(k);
  end
  if DBM and DBM.RegisterCallback then
    for k, v in pairs(DBMEventHandler) do
      DBM:RegisterCallback(k, v)
    end
  end
  frame:Show()
  
  --You know this one time, at login...
  local inInstance, instanceType = IsInInstance()
  if inInstance then
    frame:COMrequestStatus()
  end
end

function M:Unload()
  if frame then
    for k, v in pairs(frameEvents) do
      frame:UnregisterEvent(k);
    end
    frame:Hide()
  end
end

function M:SetOption(arg1, arg2)
  if arg1 == "move" then
    if moving then
      moving = false
      frame:SetMovable(false) --click-through
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
    else
      moving = true
      frame:SetMovable(true);
      frame:EnableMouse(true);
      frame:RegisterForDrag("LeftButton");
    end
    FixR:Echo("CombatTime: now "..(moving and "moving" or "fixed"))
  elseif arg1 == "reset" then
    M.O["framePoint"] = DefaultO["framePoint"]
    M.O["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    M.O["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    M.O["frameOffsetX"] = DefaultO["frameOffsetX"]
    M.O["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  elseif arg1 == "showCombatLogging" then
    local bln, updateGUI = nil, true
    if arg2 == 1 then
      bln = true
      updateGUI = false
    elseif arg2 == 0 then
      bln = false
      updateGUI = false
    else --toggle
      bln = not (M.O["showCombatLogging"] or false)
    end
    M.O["showCombatLogging"] = bln
    FixR:Echo("CombatTime: showCombatLogging: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
    if frame then
      frame.updateDisplay()
    end
    if updateGUI then
      FixR:SetModuleGUIOption(M, 1, bln)
    end
  elseif arg1 == "combatLoggingReminder" then
    local bln, updateGUI = nil, true
    if arg2 == 1 then
      bln = true
      updateGUI = false
    elseif arg2 == 0 then
      bln = false
      updateGUI = false
    else --toggle
      bln = not (M.O["combatLoggingReminder"] or false)
    end
    M.O["combatLoggingReminder"] = bln
    FixR:Echo("CombatTime: combatLoggingReminder: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
    if updateGUI then
      FixR:SetModuleGUIOption(M, 2, bln)
    end
  elseif arg1 == "continueEncounterWhenDead" then
    local bln, updateGUI = nil, true
    if arg2 == 1 then
      bln = true
      updateGUI = false
    elseif arg2 == 0 then
      bln = false
      updateGUI = false
    else --toggle
      bln = not (M.O["continueEncounterWhenDead"] or false)
    end
    M.O["continueEncounterWhenDead"] = bln
    FixR:Echo("CombatTime: continueEncounterWhenDead: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
    if updateGUI then
      FixR:SetModuleGUIOption(M, 3, bln)
    end
  elseif arg1 == "useDBMPullDetection" then
    local bln, updateGUI = nil, true
    if arg2 == 1 then
      bln = true
      updateGUI = false
    elseif arg2 == 0 then
      bln = false
      updateGUI = false
    else --toggle
      bln = not (M.O["useDBMPullDetection"] or false)
    end
    M.O["useDBMPullDetection"] = bln
    FixR:Echo("CombatTime: useDBMPullDetection: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
    if updateGUI then
      FixR:SetModuleGUIOption(M, 4, bln)
    end
  end
end
